@@ -127,10 +127,11 @@ attributes obtained through a token:
     - domain_id or project_id (depending on the token scope)
     - list of roles held for the given token scope
 
-.. note:: Some resources which are exposed by the API do not support policy
-enforcement by user_id, and only support policy enforcement by project_id.
-Some global resources do not support policy enforcement by combination of
-user_id and project_id.
+.. note::
+   Some resources which are exposed by the API do not support policy
+   enforcement by user_id, and only support policy enforcement by project_id.
+   Some global resources do not support policy enforcement by combination of
+   user_id and project_id.
 
 For example, a check on the user_id would be defined as::
 
@@ -204,10 +205,10 @@ using the :func:`~oslo_policy.policy.register` function.
 
 The following classes can be used as parents for custom special check types:
 
-    * :class:`~oslo_policy.policy.AndCheck`
-    * :class:`~oslo_policy.policy.NotCheck`
-    * :class:`~oslo_policy.policy.OrCheck`
-    * :class:`~oslo_policy.policy.RuleCheck`
+* :class:`~oslo_policy.policy.AndCheck`
+* :class:`~oslo_policy.policy.NotCheck`
+* :class:`~oslo_policy.policy.OrCheck`
+* :class:`~oslo_policy.policy.RuleCheck`
 
 Default Rule
 ~~~~~~~~~~~~
@@ -232,7 +233,6 @@ import yaml
 from oslo_policy import _cache_handler
 from oslo_policy import _checks
 from oslo_policy._i18n import _
-from oslo_policy._i18n import _LW
 from oslo_policy import _parser
 from oslo_policy import opts
 
@@ -296,9 +296,7 @@ class PolicyNotAuthorized(Exception):
     """Default exception raised for policy enforcement failure."""
 
     def __init__(self, rule, target, creds):
-        msg = (_('%(target)s is disallowed by policy rule %(rule)s '
-                 'with %(creds)s ') %
-               {'rule': rule, 'target': target, 'creds': creds})
+        msg = _("%(rule)s is disallowed by policy") % {'rule': rule}
         super(PolicyNotAuthorized, self).__init__(msg)
 
 
@@ -314,6 +312,20 @@ class PolicyNotRegistered(Exception):
         super(PolicyNotRegistered, self).__init__(msg)
 
 
+class InvalidDefinitionError(Exception):
+    def __init__(self, names):
+        msg = _('Policies %(names)s are not well defined. Check logs for '
+                'more details.') % {'names': names}
+        super(InvalidDefinitionError, self).__init__(msg)
+
+
+class InvalidRuleDefault(Exception):
+    def __init__(self, error):
+        msg = (_('Invalid policy rule default: '
+                 '%(error)s.') % {'error': error})
+        super(InvalidRuleDefault, self).__init__(msg)
+
+
 def parse_file_contents(data):
     """Parse the raw contents of a policy file.
 
@@ -321,8 +333,8 @@ def parse_file_contents(data):
     yaml or json format. Both can be parsed as yaml.
 
     :param data: A string containing the contents of a policy file.
-    :returns: A dict of of the form {'policy_name1': 'policy1',
-                                     'policy_name2': 'policy2,...}
+    :returns: A dict of the form {'policy_name1': 'policy1',
+                                  'policy_name2': 'policy2,...}
     """
     try:
         # NOTE(snikitin): jsonutils.loads() is much faster than
@@ -338,7 +350,7 @@ def parse_file_contents(data):
             # For backwards-compatibility, convert yaml error to ValueError,
             # which is what JSON loader raised.
             raise ValueError(six.text_type(e))
-    return parsed
+    return parsed or {}
 
 
 class Rules(dict):
@@ -541,7 +553,7 @@ class Enforcer(object):
             # Detect and log obvious incorrect rule definitions
             self.check_rules()
 
-    def check_rules(self):
+    def check_rules(self, raise_on_violation=False):
         """Look for rule definitions that are obviously incorrect."""
         undefined_checks = []
         cyclic_checks = []
@@ -555,11 +567,14 @@ class Enforcer(object):
                 violation = True
 
         if undefined_checks:
-            LOG.warning(_LW('Policies %(names)s reference a rule that is not '
-                        'defined.'), {'names': undefined_checks})
+            LOG.warning('Policies %(names)s reference a rule that is not '
+                        'defined.', {'names': undefined_checks})
         if cyclic_checks:
-            LOG.warning(_LW('Policies %(names)s are part of a cyclical '
-                        'reference.'), {'names': cyclic_checks})
+            LOG.warning('Policies %(names)s are part of a cyclical '
+                        'reference.', {'names': cyclic_checks})
+
+        if raise_on_violation and violation:
+            raise InvalidDefinitionError(undefined_checks + cyclic_checks)
 
         return not violation
 
@@ -822,3 +837,57 @@ class RuleDefault(object):
                  isinstance(other, self.__class__))):
             return True
         return False
+
+
+class DocumentedRuleDefault(RuleDefault):
+    """A class for holding policy-in-code policy objects definitions
+
+    This class provides the same functionality as the RuleDefault class, but it
+    also requires additional data about the policy rule being registered. This
+    is necessary so that proper documentation can be rendered based on the
+    attributes of this class. Eventually, all usage of RuleDefault should be
+    converted to use DocumentedRuleDefault.
+
+    :param operations: List of dicts containing each api url and
+        corresponding http request method.
+
+        Example::
+
+            operations=[{'path': '/foo', 'method': 'GET'},
+                        {'path': '/some', 'method': 'POST'}]
+    """
+    def __init__(self, name, check_str, description, operations):
+        super(DocumentedRuleDefault, self).__init__(
+            name, check_str, description)
+        self.operations = operations
+
+    @property
+    def description(self):
+        return self._description
+
+    @description.setter
+    def description(self, value):
+        # Validates description isn't empty.
+        if not value:
+            raise InvalidRuleDefault('Description is required')
+        self._description = value
+
+    @property
+    def operations(self):
+        return self._operations
+
+    @operations.setter
+    def operations(self, ops):
+        if not isinstance(ops, list):
+            raise InvalidRuleDefault('Operations must be a list')
+        if not ops:
+            raise InvalidRuleDefault('Operations list must not be empty')
+
+        for op in ops:
+            if 'path' not in op:
+                raise InvalidRuleDefault('Operation must contain a path')
+            if 'method' not in op:
+                raise InvalidRuleDefault('Operation must contain a method')
+            if len(op.keys()) > 2:
+                raise InvalidRuleDefault('Operation contains > 2 keys')
+        self._operations = ops
