@@ -129,6 +129,11 @@ class RulesTestCase(test_base.BaseTestCase):
         self.assertRaises(ValueError, policy.Rules.load, exemplar,
                           'default')
 
+    @mock.patch.object(_parser, 'parse_rule', lambda x: x)
+    def test_load_empty_data(self):
+        result = policy.Rules.load('', 'default')
+        self.assertEqual(result, {})
+
     @mock.patch.object(_parser, 'parse_rule', lambda x: x)
     def test_load_yaml(self):
         # Test that simplified YAML can be used with load().
@@ -724,8 +729,10 @@ class CheckFunctionTestCase(base.PolicyBaseTestCase):
         # raised.
         self.enforcer.set_rules(dict(default=_checks.FalseCheck()))
 
-        self.assertRaises(policy.PolicyNotAuthorized, self.enforcer.enforce,
-                          'rule', 'target', 'creds', True)
+        self.assertRaisesRegex(policy.PolicyNotAuthorized,
+                               " is disallowed by policy",
+                               self.enforcer.enforce,
+                               'rule', 'target', 'creds', True)
 
     def test_check_raise_custom_exception(self):
         self.enforcer.set_rules(dict(default=_checks.FalseCheck()))
@@ -834,6 +841,89 @@ class RuleDefaultTestCase(base.PolicyBaseTestCase):
         self.assertNotEqual(opt1, opt2)
 
 
+class DocumentedRuleDefaultTestCase(base.PolicyBaseTestCase):
+
+    def test_contain_operations(self):
+        opt = policy.DocumentedRuleDefault(
+            name='foo', check_str='rule:foo', description='foo_api',
+            operations=[{'path': '/foo/', 'method': 'GET'}])
+
+        self.assertEqual(1, len(opt.operations))
+
+    def test_multiple_operations(self):
+        opt = policy.DocumentedRuleDefault(
+            name='foo', check_str='rule:foo', description='foo_api',
+            operations=[{'path': '/foo/', 'method': 'GET'},
+                        {'path': '/foo/', 'method': 'POST'}])
+
+        self.assertEqual(2, len(opt.operations))
+
+    def test_description_not_empty(self):
+        invalid_desc = ''
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description=invalid_desc,
+                          operations=[{'path': '/foo/', 'method': 'GET'}])
+
+    def test_operation_not_empty_list(self):
+        invalid_op = []
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description='foo_api',
+                          operations=invalid_op)
+
+    def test_operation_must_be_list(self):
+        invalid_op = 'invalid_op'
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description='foo_api',
+                          operations=invalid_op)
+
+    def test_operation_must_be_list_of_dicts(self):
+        invalid_op = ['invalid_op']
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description='foo_api',
+                          operations=invalid_op)
+
+    def test_operation_must_have_path(self):
+        invalid_op = [{'method': 'POST'}]
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description='foo_api',
+                          operations=invalid_op)
+
+    def test_operation_must_have_method(self):
+        invalid_op = [{'path': '/foo/path/'}]
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description='foo_api',
+                          operations=invalid_op)
+
+    def test_operation_must_contain_method_and_path_only(self):
+        invalid_op = [{'path': '/some/path/',
+                       'method': 'GET',
+                       'break': 'me'}]
+        self.assertRaises(policy.InvalidRuleDefault,
+                          policy.DocumentedRuleDefault,
+                          name='foo',
+                          check_str='rule:foo',
+                          description='foo_api',
+                          operations=invalid_op)
+
+
 class EnforcerCheckRulesTest(base.PolicyBaseTestCase):
     def setUp(self):
         super(EnforcerCheckRulesTest, self).setUp()
@@ -841,7 +931,7 @@ class EnforcerCheckRulesTest(base.PolicyBaseTestCase):
     def test_no_violations(self):
         self.create_config_file('policy.json', POLICY_JSON_CONTENTS)
         self.enforcer.load_rules(True)
-        self.assertTrue(self.enforcer.check_rules())
+        self.assertTrue(self.enforcer.check_rules(raise_on_violation=True))
 
     def test_undefined_rule(self):
         rules = jsonutils.dumps({'foo': 'rule:bar'})
@@ -850,6 +940,14 @@ class EnforcerCheckRulesTest(base.PolicyBaseTestCase):
 
         self.assertFalse(self.enforcer.check_rules())
 
+    def test_undefined_rule_raises(self):
+        rules = jsonutils.dumps({'foo': 'rule:bar'})
+        self.create_config_file('policy.json', rules)
+        self.enforcer.load_rules(True)
+
+        self.assertRaises(policy.InvalidDefinitionError,
+                          self.enforcer.check_rules, raise_on_violation=True)
+
     def test_cyclical_rules(self):
         rules = jsonutils.dumps({'foo': 'rule:bar', 'bar': 'rule:foo'})
         self.create_config_file('policy.json', rules)
@@ -857,6 +955,14 @@ class EnforcerCheckRulesTest(base.PolicyBaseTestCase):
 
         self.assertFalse(self.enforcer.check_rules())
 
+    def test_cyclical_rules_raises(self):
+        rules = jsonutils.dumps({'foo': 'rule:bar', 'bar': 'rule:foo'})
+        self.create_config_file('policy.json', rules)
+        self.enforcer.load_rules(True)
+
+        self.assertRaises(policy.InvalidDefinitionError,
+                          self.enforcer.check_rules, raise_on_violation=True)
+
     def test_complex_cyclical_rules_false(self):
         rules = jsonutils.dumps({'foo': 'rule:bar',
                                  'bar': 'rule:baz and role:admin',
